@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.174 2007/12/10 16:07:30 aland Exp $
+ * Version:	$Id: auth.c,v 1.186 2008/06/05 10:20:43 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: auth.c,v 1.174 2007/12/10 16:07:30 aland Exp $")
+RCSID("$Id: auth.c,v 1.186 2008/06/05 10:20:43 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -49,7 +49,7 @@ char *auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli)
 	snprintf(buf, buflen, "from client %.128s port %u%s%.128s%s",
 			request->client->shortname, port,
 		 (do_cli ? " cli " : ""), (do_cli ? (char *)cli->vp_strvalue : ""),
-		 (request->packet->src_port == 0) ? " via TLS tunnel" : "");
+		 (request->packet->dst_port == 0) ? " via TLS tunnel" : "");
 
 	return buf;
 }
@@ -188,7 +188,11 @@ static int rad_check_password(REQUEST *request)
 	 *  that means it is accepted and we do no further
 	 *  authentication
 	 */
-	if ((auth_type == PW_AUTHTYPE_ACCEPT) || (request->proxy)) {
+	if ((auth_type == PW_AUTHTYPE_ACCEPT)
+#ifdef WITH_PROXY
+	    || (request->proxy)
+#endif
+	    ) {
 		DEBUG2("  rad_check_password: Auth-Type = Accept, accepting the user");
 		return 0;
 	}
@@ -446,20 +450,20 @@ int rad_postauth(REQUEST *request)
 int rad_authenticate(REQUEST *request)
 {
 	VALUE_PAIR	*namepair;
+#ifdef WITH_SESSION_MGMT
 	VALUE_PAIR	*check_item;
-	VALUE_PAIR	*auth_item;
+#endif
+	VALUE_PAIR	*auth_item = NULL;
 	VALUE_PAIR	*module_msg;
 	VALUE_PAIR	*tmp = NULL;
 	int		result;
-	char		umsg[MAX_STRING_LEN + 1];
-	const char	*user_msg = NULL;
 	const char	*password;
-	char		logstr[1024];
 	char		autz_retry = 0;
 	int		autz_type = 0;
 
 	password = "";
 
+#ifdef WITH_PROXY
 	/*
 	 *	If this request got proxied to another server, we need
 	 *	to check whether it authenticated the request or not.
@@ -475,7 +479,8 @@ int rad_authenticate(REQUEST *request)
 						&request->config_items,
 						PW_AUTH_TYPE, PW_TYPE_INTEGER);
 			if (tmp) tmp->vp_integer = PW_AUTHTYPE_ACCEPT;
-			break;
+			goto authenticate;
+
 		/*
 		 *	Challenges are punted back to the NAS without any
 		 *	further processing.
@@ -498,6 +503,7 @@ int rad_authenticate(REQUEST *request)
 			return RLM_MODULE_REJECT;
 		}
 	}
+#endif
 
 	/*
 	 *	Get the username from the request.
@@ -585,26 +591,41 @@ autz_redo:
 	 *	modules has decided that a proxy should be used. If
 	 *	so, get out of here and send the packet.
 	 */
-	if ((request->proxy == NULL) &&
+	if (
+#ifdef WITH_PROXY
+	    (request->proxy == NULL) &&
+#endif
 	    ((tmp = pairfind(request->config_items, PW_PROXY_TO_REALM)) != NULL)) {
 		REALM *realm;
 
+		realm = realm_find2(tmp->vp_strvalue);
+
 		/*
-		 *	Catch users who set Proxy-To-Realm to a LOCAL
-		 *	realm (sigh).
+		 *	Don't authenticate, as the request is going to
+		 *	be proxied.
 		 */
-		realm = realm_find(tmp->vp_strvalue);
-		if (realm && !realm->auth_pool) {
-			DEBUG2("  WARNING: You set Proxy-To-Realm = %s, but it is a LOCAL realm!  Cancelling invalid proxy request.", realm->name);
-		} else {
-			/*
-			 *	Don't authenticate, as the request is
-			 *	proxied.
-			 */
+		if (realm && realm->auth_pool) {
 			return RLM_MODULE_OK;
 		}
+
+		/*
+		 *	Catch users who set Proxy-To-Realm to a LOCAL
+		 *	realm (sigh).  But don't complain if it is
+		 *	*the* LOCAL realm.
+		 */
+		if (realm &&(strcmp(realm->name, "LOCAL") != 0)) {
+			DEBUG2("  WARNING: You set Proxy-To-Realm = %s, but it is a LOCAL realm!  Cancelling invalid proxy request.", realm->name);
+		}
+
+		if (!realm) {
+			DEBUG2("  WARNING: You set Proxy-To-Realm = %s, but the realm does not exist!  Cancelling invalid proxy request.", tmp->vp_strvalue);
+		}
 	}
 
+#ifdef WITH_PROXY
+ authenticate:
+#endif
+
 	/*
 	 *	Perhaps there is a Stripped-User-Name now.
 	 */
@@ -658,9 +679,13 @@ autz_redo:
 		}
 	}
 
+#ifdef WITH_SESSION_MGMT
 	if (result >= 0 &&
 	    (check_item = pairfind(request->config_items, PW_SIMULTANEOUS_USE)) != NULL) {
 		int r, session_type = 0;
+		char		logstr[1024];
+		char		umsg[MAX_STRING_LEN + 1];
+		const char	*user_msg = NULL;
 
 		tmp = pairfind(request->config_items, PW_SESSION_TYPE);
 		if (tmp) {
@@ -716,6 +741,7 @@ autz_redo:
 			}
 		}
 	}
+#endif
 
 	/*
 	 *	Result should be >= 0 here - if not, it means the user
@@ -725,15 +751,6 @@ autz_redo:
 		return RLM_MODULE_REJECT;
 	}
 
-	/*
-	 *	We might need this later.  The 'password' string
-	 *	is NOT used anywhere below here, except for logging,
-	 *	so it should be safe...
-	 */
-	if ((auth_item != NULL) && (auth_item->attribute == PW_CHAP_PASSWORD)) {
-		password = "CHAP-Password";
-	}
-
 	/*
 	 *	Add the port number to the Framed-IP-Address if
 	 *	vp->addport is set.
