@@ -1,7 +1,7 @@
 /*
  * evaluate.c	Evaluate complex conditions
  *
- * Version:	$Id: evaluate.c,v 1.42 2008/05/02 09:11:18 aland Exp $
+ * Version:	$Id: evaluate.c,v 1.46 2008/06/05 10:32:54 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: evaluate.c,v 1.42 2008/05/02 09:11:18 aland Exp $")
+RCSID("$Id: evaluate.c,v 1.46 2008/06/05 10:32:54 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -48,6 +48,16 @@ RCSID("$Id: evaluate.c,v 1.42 2008/05/02 09:11:18 aland Exp $")
 #endif
 
 
+#ifndef NDEBUG
+#ifndef DEBUG4
+#define DEBUG4  if (debug_flag > 4)log_debug
+#endif
+#else
+#define DEBUG4 if (0) log_debug
+#endif
+
+#ifdef WITH_UNLANG
+
 static int all_digits(const char *string)
 {
 	const char *p = string;
@@ -59,14 +69,6 @@ static int all_digits(const char *string)
 	return (*p == '\0');
 }
 
-#ifndef NDEBUG
-#ifndef DEBUG4
-#define DEBUG4  if (debug_flag > 4)log_debug
-#endif
-#else
-#define DEBUG4 if (0) log_debug
-#endif
-
 static const char *filler = "????????????????????????????????????????????????????????????????";
 
 static const char *expand_string(char *buffer, size_t sizeof_buffer,
@@ -233,6 +235,7 @@ static int radius_get_vp(REQUEST *request, const char *name, VALUE_PAIR **vp_p)
 		vp_name += 6;
 		vps = myrequest->reply->vps;
 
+#ifdef WITH_PROXY
 	} else if (strncmp(vp_name, "proxy-request:", 14) == 0) {
 		vp_name += 14;
 		if (request->proxy) vps = myrequest->proxy->vps;
@@ -240,6 +243,7 @@ static int radius_get_vp(REQUEST *request, const char *name, VALUE_PAIR **vp_p)
 	} else if (strncmp(vp_name, "proxy-reply:", 12) == 0) {
 		vp_name += 12;
 		if (request->proxy_reply) vps = myrequest->proxy_reply->vps;
+#endif
 
 	} else if (strncmp(vp_name, "config:", 7) == 0) {
 		vp_name += 7;
@@ -275,7 +279,11 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 	int result;
 	int lint, rint;
 	VALUE_PAIR *vp = NULL;
+#ifdef HAVE_REGEX_H
 	char buffer[1024];
+#else
+	cflags = cflags;	/* -Wunused */
+#endif
 
 	rt = rt;		/* -Wunused */
 
@@ -311,6 +319,24 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 			}
 
 			if (!vp) {
+				DICT_ATTR *da;
+				
+				/*
+				 *	The attribute on the LHS may
+				 *	have been a dynamically
+				 *	registered callback.  i.e. it
+				 *	doesn't exist as a VALUE_PAIR.
+				 *	If so, try looking for it.
+				 */
+				da = dict_attrbyname(pleft);
+				if (da && radius_find_compare(da->attr)) {
+					VALUE_PAIR *check = pairmake(pleft, pright, token);
+					*presult = (radius_callback_compare(request, NULL, check, NULL, NULL) == 0);
+					pairfree(&check);
+					if (*presult)  return TRUE;
+					return FALSE;
+				}
+				
 				DEBUG2("    (Attribute %s was not found)",
 				       pleft);
 				return FALSE;
@@ -339,10 +365,12 @@ static int radius_do_cmp(REQUEST *request, int *presult,
 			myvp.operator = token;
 			*presult = paircmp(&myvp, vp);
 			return TRUE;
-		} /* else it's not a attribute in the dictionary */
+		} /* else it's not a VP in a list */
 	}
 
+#ifdef HAVE_REGEX_H
 	do_checks:
+#endif
 	switch (token) {
 	case T_OP_GE:
 	case T_OP_GT:
@@ -499,10 +527,8 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 	char left[1024], right[1024], comp[4];
 	const char *pleft, *pright;
 	char  xleft[1024], xright[1024];
-#ifdef HAVE_REGEX_H
 	int cflags = 0;
-#endif
-
+	
 	if (!ptr || !*ptr || (depth >= 64)) {
 		radlog(L_ERR, "Internal sanity check failed in evaluate condition");
 		return FALSE;
@@ -794,7 +820,7 @@ int radius_evaluate_condition(REQUEST *request, int modreturn, int depth,
 	if (evaluate_next_condition) *presult = result;
 	return TRUE;
 }
-
+#endif
 
 static void fix_up(REQUEST *request)
 {
@@ -817,7 +843,6 @@ static void fix_up(REQUEST *request)
 	}
 }
 
-
 /*
  *	The pairmove() function in src/lib/valuepair.c does all sorts of
  *	extra magic that we don't want here.
@@ -1091,6 +1116,7 @@ void radius_pairmove(REQUEST *request, VALUE_PAIR **to, VALUE_PAIR *from)
 }
 
 
+#ifdef WITH_UNLANG
 /*
  *     Copied shamelessly from conffile.c, to simplify the API for
  *     now...
@@ -1147,11 +1173,13 @@ int radius_update_attrlist(REQUEST *request, CONF_SECTION *cs,
 	} else if (strcmp(name, "reply") == 0) {
 		output_vps = &request_vps->reply->vps;
 
+#ifdef WITH_PROXY
 	} else if (strcmp(name, "proxy-request") == 0) {
 		if (request->proxy) output_vps = &request_vps->proxy->vps;
 
 	} else if (strcmp(name, "proxy-reply") == 0) {
 		if (request->proxy_reply) output_vps = &request->proxy_reply->vps;
+#endif
 
 	} else if (strcmp(name, "config") == 0) {
 		output_vps = &request_vps->config_items;
@@ -1217,3 +1245,4 @@ int radius_update_attrlist(REQUEST *request, CONF_SECTION *cs,
 
 	return RLM_MODULE_UPDATED;
 }
+#endif
