@@ -1,7 +1,7 @@
 /*
  * radclient.c	General radius packet debug tool.
  *
- * Version:	$Id: radclient.c,v 1.118 2007/12/17 12:30:42 aland Exp $
+ * Version:	$Id: radclient.c,v 1.120 2008/04/03 13:43:12 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: radclient.c,v 1.118 2007/12/17 12:30:42 aland Exp $")
+RCSID("$Id: radclient.c,v 1.120 2008/04/03 13:43:12 aland Exp $")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/conf.h>
@@ -832,6 +832,7 @@ int main(int argc, char **argv)
 
 		case 'q':
 			do_output = 0;
+			fr_log_fp = NULL; /* no output from you, either! */
 			break;
 		case 'r':
 			if (!isdigit((int) *optarg))
@@ -876,11 +877,12 @@ int main(int argc, char **argv)
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("radclient: $Id: radclient.c,v 1.118 2007/12/17 12:30:42 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
+			printf("radclient: $Id: radclient.c,v 1.120 2008/04/03 13:43:12 aland Exp $ built on " __DATE__ " at " __TIME__ "\n");
 			exit(0);
 			break;
 		case 'x':
 			librad_debug++;
+			fr_log_fp = stdout;
 			break;
 		case 'h':
 		default:
