@@ -19,7 +19,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: rlm_ldap.c,v 1.191 2008/01/05 18:51:50 nbk Exp $")
+RCSID("$Id: rlm_ldap.c,v 1.195 2008/05/08 16:07:54 aland Exp $")
 
 #include <freeradius-devel/radiusd.h>
 #include <freeradius-devel/modules.h>
@@ -125,8 +125,8 @@ typedef struct {
 	char           *server;
 	int             port;
 	int             timelimit;
-	struct timeval  net_timeout;
-	struct timeval  timeout;
+	int  		net_timeout;
+	int		timeout;
 	int             debug;
 	int             tls_mode;
 	int		start_tls;
@@ -160,6 +160,7 @@ typedef struct {
 #endif
 	int             ldap_debug; /* Debug flag for LDAP SDK */
 	char		*xlat_name; /* name used to xlat */
+	char		*auth_type;
 	char		*tls_cacertfile;
 	char		*tls_cacertdir;
 	char		*tls_certfile;
@@ -208,10 +209,10 @@ static const CONF_PARSER module_config[] = {
 	 */
 	/* wait forever on network activity */
 	{"net_timeout", PW_TYPE_INTEGER,
-	 offsetof(ldap_instance,net_timeout.tv_sec), NULL, "10"},
+	 offsetof(ldap_instance,net_timeout), NULL, "10"},
 	/* wait forever for search results */
 	{"timeout", PW_TYPE_INTEGER,
-	 offsetof(ldap_instance,timeout.tv_sec), NULL, "20"},
+	 offsetof(ldap_instance,timeout), NULL, "20"},
 	/* allow server unlimited time for search (server-side limit) */
 	{"timelimit", PW_TYPE_INTEGER,
 	 offsetof(ldap_instance,timelimit), NULL, "20"},
@@ -404,8 +405,6 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 #endif
 	}
 
-	inst->timeout.tv_usec = 0;
-	inst->net_timeout.tv_usec = 0;
 	/* workaround for servers which support LDAPS but not START TLS */
 	if(inst->port == LDAPS_PORT || inst->tls_mode)
 		inst->tls_mode = LDAP_OPT_X_TLS_HARD;
@@ -458,9 +457,17 @@ ldap_instantiate(CONF_SECTION * conf, void **instance)
 	 */
 	if (inst->set_auth_type) {
 		DICT_VALUE *dv = dict_valbyname(PW_AUTH_TYPE, xlat_name);
+
+		/*
+		 *	No section of *my* name, but maybe there's an
+		 *	LDAP section...
+		 */
+		if (!dv) dv = dict_valbyname(PW_AUTH_TYPE, "LDAP");
 		if (!dv) {
-			DEBUG2("rlm_ldap: Over-riding set_auth_type, as we're not listed in the \"authenticate\" section.");
+			DEBUG2("rlm_ldap: Over-riding set_auth_type, as there is no module %s listed in the \"authenticate\" section.", xlat_name);
 			inst->set_auth_type = 0;
+		} else {
+			inst->auth_type = dv->name; /* doesn't change on HUP */
 		}
 	} /* else no need to look up the value */
 
@@ -753,6 +760,7 @@ static int perform_search(void *instance, LDAP_CONN *conn,
 	int		ldap_errno = 0;
 	ldap_instance  *inst = instance;
 	int		search_retry = 0;
+	struct timeval  tv;
 
 	*result = NULL;
 
@@ -784,10 +792,13 @@ retry:
 		conn->bound = 1;
 		conn->failed_conns = 0;
 	}
+
+	tv.tv_sec = inst->timeout;
+	tv.tv_usec = 0;
 	DEBUG2("rlm_ldap: performing search in %s, with filter %s",
 	       search_basedn ? search_basedn : "(null)" , filter);
 	switch (ldap_search_st(conn->ld, search_basedn, scope, filter,
-			       attrs, 0, &(inst->timeout), result)) {
+			       attrs, 0, &tv, result)) {
 	case LDAP_SUCCESS:
 	case LDAP_NO_SUCH_OBJECT:
 		break;
@@ -1694,8 +1705,8 @@ static int ldap_authorize(void *instance, REQUEST * request)
 	    request->password &&
 	    (request->password->attribute == PW_USER_PASSWORD) &&
 	    !added_known_password) {
-		pairadd(check_pairs, pairmake("Auth-Type", inst->xlat_name, T_OP_EQ));
-		DEBUG("rlm_ldap: Setting Auth-Type = %s", inst->xlat_name);
+		pairadd(check_pairs, pairmake("Auth-Type", inst->auth_type, T_OP_EQ));
+		DEBUG("rlm_ldap: Setting Auth-Type = %s", inst->auth_type);
 	}
 
 	DEBUG("rlm_ldap: user %s authorized to use remote access",
@@ -2082,7 +2093,7 @@ static int ldap_postauth(void *instance, REQUEST * request)
 					return RLM_MODULE_FAIL;
 				}
 
-				vp_fdn = pairfind(request->packet->vps, da->attr);
+				vp_fdn = pairfind(request->config_items, da->attr);
 				if (vp_fdn == NULL) {
 					DEBUG("rlm_ldap: User's FQDN not in config items list.");
 					return RLM_MODULE_FAIL;
@@ -2153,6 +2164,7 @@ static LDAP *ldap_connect(void *instance, const char *dn, const char *password,
 	int             msgid, rc, ldap_version;
 	int		ldap_errno = 0;
 	LDAPMessage    *res;
+	struct timeval tv;
 
 	if (inst->is_url){
 #ifdef HAVE_LDAP_INITIALIZE
@@ -2171,9 +2183,11 @@ static LDAP *ldap_connect(void *instance, const char *dn, const char *password,
 			return (NULL);
 		}
 	}
+	tv.tv_sec = inst->net_timeout;
+	tv.tv_usec = 0;
 	if (ldap_set_option(ld, LDAP_OPT_NETWORK_TIMEOUT,
-			    (void *) &(inst->net_timeout)) != LDAP_OPT_SUCCESS) {
-		radlog(L_ERR, "rlm_ldap: Could not set LDAP_OPT_NETWORK_TIMEOUT %ld.%ld", inst->net_timeout.tv_sec, inst->net_timeout.tv_usec);
+			    (void *) &tv) != LDAP_OPT_SUCCESS) {
+		radlog(L_ERR, "rlm_ldap: Could not set LDAP_OPT_NETWORK_TIMEOUT %d", inst->net_timeout);
 	}
 
 	if (ldap_set_option(ld, LDAP_OPT_TIMELIMIT,
@@ -2320,7 +2334,9 @@ static LDAP *ldap_connect(void *instance, const char *dn, const char *password,
 	}
 	DEBUG("rlm_ldap: waiting for bind result ...");
 
-	rc = ldap_result(ld, msgid, 1, &(inst->timeout), &res);
+	tv.tv_sec = inst->timeout;
+	tv.tv_usec = 0;
+	rc = ldap_result(ld, msgid, 1, &tv, &res);
 
 	if (rc < 1) {
 		DEBUG("rlm_ldap: ldap_result()");
