@@ -21,10 +21,12 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: smbencrypt.c,v 1.13 2007/11/25 14:02:11 aland Exp $")
+RCSID("$Id: smbencrypt.c,v 1.14 2008/06/05 12:05:24 aland Exp $")
 
 #include	<freeradius-devel/libradius.h>
 #include        <freeradius-devel/md4.h>
+#include        <freeradius-devel/md5.h>
+#include        <freeradius-devel/sha1.h>
 #include        <ctype.h>
 
 
